@@ -22,17 +22,26 @@ import uuid
 
 from twisted.internet.defer import Deferred
 
-from ubuntuone.storageprotocol import protocol_pb2, sharersp
+from ubuntuone.storageprotocol import protocol_pb2, sharersp, delta, request
 from ubuntuone.storageprotocol.client import (
-    StorageClient, CreateUDF, ListVolumes, DeleteVolume
+    StorageClient, CreateUDF, ListVolumes, DeleteVolume, GetDelta,
+    Authenticate, MakeFile, MakeDir, PutContent, Move, Unlink
 )
 from ubuntuone.storageprotocol import volumes
+from tests import test_delta_info
+
+# let's not get picky about aatributes outside __init__ in tests
+# pylint: disable-msg=W0201
+# it's ok to access internals in the test suite
+# pylint: disable-msg=W0212
 
 PATH = u'~/Documents/pdfs/moño/'
 NAME = u'UDF-me'
 VOLUME = uuid.UUID('12345678-1234-1234-1234-123456789abc')
+SHARE = uuid.UUID('33333333-1234-1234-1234-123456789abc')
 NODE = uuid.UUID('FEDCBA98-7654-3211-2345-6789ABCDEF12')
 USER = u'Dude'
+GENERATION = 999
 
 class FakedError(Exception):
     """Stub to replace Request.error."""
@@ -96,6 +105,22 @@ def set_share_message(message):
     message.share.accepted = False
     message.share.access_level = 0
 
+class MethodMock(object):
+    """A class to overwrite methods to know if they were called.
+
+    @ivar called: boolean, true if the class was called
+    @ivar call_count: int, the number of calls
+    """
+
+    def __init__(self):
+        """Create the mock."""
+        self.called = False
+        self.call_count = 0
+
+    def __call__(self):
+        """Update call stats."""
+        self.called = True
+        self.call_count += 1
 
 class DummyAttribute(object):
     """Helper class to replace non-related classes."""
@@ -133,6 +158,36 @@ class ClientTestCase(unittest.TestCase):
         self.client = None
 
     # client to server
+    def test_client_get_delta(self):
+        """Get a delta."""
+        original = GetDelta.start
+        GetDelta.start = was_called(self, 'called')
+
+        try:
+            result = self.client.get_delta(share_id=SHARE, from_generation=0)
+            self.assertTrue(self.called, 'GetDelta.start() was called')
+            self.assertTrue(isinstance(result, Deferred))
+        finally:
+            GetDelta.start = original
+
+    def test_client_get_delta_from_scratch(self):
+        """Get a delta from scratch."""
+        original = GetDelta.start
+        GetDelta.start = was_called(self, 'called')
+
+        try:
+            result = self.client.get_delta(share_id=SHARE, from_scratch=True)
+            self.assertTrue(self.called, 'GetDelta.start() was called')
+            self.assertTrue(isinstance(result, Deferred))
+        finally:
+            GetDelta.start = original
+
+    def test_client_get_delta_bad(self):
+        """Require from_generation or from_scratch."""
+
+        self.assertRaises(TypeError, self.client.get_delta,
+                          share_id=SHARE, callback=1)
+
     def test_create_udf(self):
         """Test create_udf."""
         original = CreateUDF.start
@@ -173,7 +228,6 @@ class ClientTestCase(unittest.TestCase):
         """Test callback setting."""
         a_callback = lambda x: None
         self.client.set_volume_deleted_callback(a_callback)
-        # pylint: disable-msg= W0212
         self.assertTrue(self.client._volume_deleted_callback is a_callback)
 
     def test_callback_must_be_callable(self):
@@ -184,13 +238,21 @@ class ClientTestCase(unittest.TestCase):
         self.assertRaises(TypeError, self.client.set_volume_deleted_callback,
                           'world')
 
+        self.assertRaises(TypeError,
+                          self.client.set_volume_new_generation_callback, 'fu')
+
     def test_set_volume_created_callback(self):
         """Test callback setting."""
         a_callback = lambda y, z: None
         self.client.set_volume_created_callback(a_callback)
-        # pylint: disable-msg= W0212
         self.assertTrue(self.client._volume_created_callback is a_callback)
 
+    def test_set_volume_new_generation_callback(self):
+        """Test callback setting."""
+        cback = lambda y, z: None
+        self.client.set_volume_new_generation_callback(cback)
+        self.assertTrue(self.client._volume_new_generation_callback is cback)
+
     # share notification callbacks
     def test_share_change_callback(self):
         """Test share_change callback usage."""
@@ -249,6 +311,41 @@ class ClientTestCase(unittest.TestCase):
         self.assertEquals(self.answer[0], share_id)
         self.assertEquals(self.answer[1], "Yes")
 
+    def test_handle_volume_new_generation_uuid(self):
+        """Test handle_VOLUME_NEW_GENERATION with an uuid id."""
+        # set the callback to record the info
+        called = []
+        f = lambda *a: called.append(a)
+        self.client.set_volume_new_generation_callback(f)
+
+        # create the message
+        message = protocol_pb2.Message()
+        message.type = protocol_pb2.Message.VOLUME_NEW_GENERATION
+        volume_id = uuid.uuid4()
+        message.volume_new_generation.volume = str(volume_id)
+        message.volume_new_generation.generation = 77
+
+        # send the message, and assert the callback is called with good info
+        self.client.handle_VOLUME_NEW_GENERATION(message)
+        self.assertEqual(called[0], (volume_id, 77))
+
+    def test_handle_volume_new_generation_root(self):
+        """Test handle_VOLUME_NEW_GENERATION for ROOT."""
+        # set the callback to record the info
+        called = []
+        f = lambda *a: called.append(a)
+        self.client.set_volume_new_generation_callback(f)
+
+        # create the message
+        message = protocol_pb2.Message()
+        message.type = protocol_pb2.Message.VOLUME_NEW_GENERATION
+        message.volume_new_generation.volume = request.ROOT
+        message.volume_new_generation.generation = 77
+
+        # send the message, and assert the callback is called with good info
+        self.client.handle_VOLUME_NEW_GENERATION(message)
+        self.assertEqual(called[0], (request.ROOT, 77))
+
     # server to client
     def test_handle_volume_created(self):
         """Test handle_VOLUME_CREATED."""
@@ -520,6 +617,180 @@ class DeleteVolumeTestCase(unittest.TestCase):
 
         self.assertTrue(self.done_called, 'done() was called')
 
+class GetDeltaTestCase(unittest.TestCase):
+    """Test cases for GetDelta op."""
+
+    def setUp(self):
+        """Initialize testing protocol."""
+        self.protocol = FakedProtocol()
+        self.request = GetDelta(self.protocol, SHARE, 0)
+        self.request.error = faked_error
+        self.done_called = False
+        self.request.done = was_called(self, 'done_called')
+
+    def tearDown(self):
+        """Clean up."""
+        self.protocol = None
+        self.request = None
+
+    def test_init(self):
+        """Test request creation."""
+        self.assertEquals(str(SHARE), self.request.share_id)
+
+    def test_start(self):
+        """Test request start."""
+        self.request.start()
+
+        self.assertEquals(1, len(self.request.protocol.messages))
+        actual_msg, = self.request.protocol.messages
+        self.assertEquals(protocol_pb2.Message.GET_DELTA, actual_msg.type)
+        self.assertEquals(self.request.share_id,
+                          actual_msg.get_delta.share)
+
+    def test_process_message_error(self):
+        """Test request processMessage on error."""
+        message = protocol_pb2.Message()
+        self.assertRaises(FakedError, self.request.processMessage, message)
+
+    def test_process_message_ok(self):
+        """Test request processMessage on sucess."""
+        message = protocol_pb2.Message()
+        message.type = protocol_pb2.Message.DELTA_END
+        message.delta_end.generation = 100
+        message.delta_end.full = True
+        message.delta_end.free_bytes = 200
+        self.request.processMessage(message)
+
+        self.assertTrue(self.done_called, 'done() was called')
+        self.assertEquals(self.request.end_generation,
+                          message.delta_end.generation)
+        self.assertEquals(self.request.full, message.delta_end.full)
+        self.assertEquals(self.request.free_bytes,
+                          message.delta_end.free_bytes)
+
+    def test_process_message_content(self):
+        """Test request processMessage for content."""
+        message = test_delta_info.get_message()
+        self.request.processMessage(message)
+        self.assertTrue(delta.from_message(message) in self.request.response)
+
+    def test_process_message_content_twice(self):
+        """Test request processMessage for content."""
+        message = test_delta_info.get_message()
+        self.request.processMessage(message)
+        message = test_delta_info.get_message()
+        self.request.processMessage(message)
+        self.assertEqual(len(self.request.response), 2)
+
+    def test_process_message_content_callback(self):
+        """Test request processMessage for content w/callback."""
+        response = []
+        self.request = GetDelta(self.protocol, SHARE, 0,
+                                callback=response.append)
+        message = test_delta_info.get_message()
+        self.request.processMessage(message)
+        self.assertTrue(delta.from_message(message) in response)
+
+    def test_from_scratch_flag(self):
+        """Test from scratch flag."""
+        self.request = GetDelta(self.protocol, SHARE, 0, from_scratch=True)
+        self.request.done = was_called(self, 'done_called')
+        self.request.start()
+
+        self.assertEquals(1, len(self.request.protocol.messages))
+        actual_msg, = self.request.protocol.messages
+        self.assertEquals(protocol_pb2.Message.GET_DELTA, actual_msg.type)
+        self.assertEquals(self.request.share_id,
+                          actual_msg.get_delta.share)
+
+class TestAuth(unittest.TestCase):
+    """Tests the authentication request."""
+
+    def test_session_id(self):
+        """Test that the request has the session id attribute."""
+
+        SESSION_ID = "opaque_session_id"
+        request = Authenticate(FakedProtocol(), None)
+        request.done = MethodMock()
+        message = protocol_pb2.Message()
+        message.type = protocol_pb2.Message.AUTH_AUTHENTICATED
+        message.session_id = SESSION_ID
+        request.processMessage(message)
+        self.assert_(request.done.called)
+        self.assertEqual(request.session_id, SESSION_ID)
+
+
+class TestGenerationInRequests(unittest.TestCase):
+    """Base class for testing that actions that change the volume will
+    have a new_generation attribute set."""
+
+    def build_request(self):
+        """Creates the request object."""
+        return MakeFile(FakedProtocol(), None, None, "name")
+
+    def build_message(self):
+        """Creates the ending message for the request."""
+        message = protocol_pb2.Message()
+        message.type = protocol_pb2.Message.NEW_FILE
+        message.new_generation = GENERATION
+        return message
+
+    def test_make(self):
+        """Test the request for new_generation."""
+        request = self.build_request()
+        request.done = MethodMock()
+        message = self.build_message()
+        request.processMessage(message)
+        self.assert_(request.done.called)
+        self.assertEqual(request.new_generation, GENERATION)
+
+
+class TestGenerationInRequestsMakeDir(TestGenerationInRequests):
+    """Tests for new_generation in MakeDir."""
+
+    def build_request(self):
+        """Creates the request object."""
+        return MakeDir(FakedProtocol(), None, None, "name")
+
+    def build_message(self):
+        """Creates the ending message for the request."""
+        message = protocol_pb2.Message()
+        message.type = protocol_pb2.Message.NEW_DIR
+        message.new_generation = GENERATION
+        return message
+
+
+class TestGenerationInRequestsPutContent(TestGenerationInRequests):
+    """Tests for new_generation in PutContent."""
+
+    def build_request(self):
+        """Creates the request object."""
+        return PutContent(FakedProtocol(), None, None, None, None,
+                          None, None, None, None)
+
+    def build_message(self):
+        """Creates the ending message for the request."""
+        message = protocol_pb2.Message()
+        message.type = protocol_pb2.Message.OK
+        message.new_generation = GENERATION
+        return message
+
+
+class TestGenerationInRequestsUnlink(TestGenerationInRequestsPutContent):
+    """Tests for new_generation in Unlink."""
+
+    def build_request(self):
+        """Creates the request object."""
+        return Unlink(FakedProtocol(), None, None)
+
+
+class TestGenerationInRequestsMove(TestGenerationInRequestsPutContent):
+    """Tests for new_generation in Move."""
+
+    def build_request(self):
+        """Creates the request object."""
+        return Move(FakedProtocol(), None, None, None, None)
+
 
 if __name__ == '__main__':
     unittest.main()
