@@ -21,7 +21,7 @@
 
 from __future__ import with_statement
 
-import unittest
+import uuid
 
 from twisted.trial.unittest import TestCase as TwistedTestCase
 from twisted.internet.defer import inlineCallbacks
@@ -29,14 +29,27 @@ from twisted.python.failure import Failure
 
 from ubuntuone.storageprotocol import errors, protocol_pb2
 from ubuntuone.storageprotocol.request import (
-    RequestHandler, Request)
+    RequestHandler, Request, RequestResponse
+)
+
+# let's not get picky about attributes outside __init__ in tests
+# pylint: disable-msg=W0201
 
 
 class MindlessRequest(Request):
-    """A mindless request which never actually does anything."""
+    """A mindless Request which never actually does anything."""
 
     def _start(self):
         """Does nothing."""
+        return getattr(self, 'start_result', None)
+
+
+class MindlessRequestResponse(RequestResponse):
+    """A mindless RequestResponse which never actually does anything."""
+
+    def _start(self):
+        """Does nothing."""
+        return getattr(self, 'start_result', None)
 
 
 class ConnectionLike(object):
@@ -55,48 +68,45 @@ class ConnectionLike(object):
         self.producer = None
 
 
-class TestRequests(TwistedTestCase):
-    """Tests for request handling."""
+class TestRequest(TwistedTestCase):
+    """Tests for Request."""
 
-    @inlineCallbacks
-    def test_disconnect_aborts_requests(self):
-        """Test that disconnection aborts outstanding requests."""
-
-
-        class OurException(RuntimeError):
-            """An exception class to look for."""
-
-
-        transport = ConnectionLike()
-        protocol = RequestHandler()
-        protocol.makeConnection(transport)
-        request = MindlessRequest(protocol)
-        request.start()
-        protocol.connectionLost(Failure(OurException()))
-        try:
-            yield request.deferred
-        except OurException:
-            pass # passed
-        else:
-            self.fail("Expected to fail with the correct reason.")
-
-
-class TestRequest(unittest.TestCase):
-    """Tests for request object."""
+    timeout = 2
 
     def setUp(self):
         """Init."""
-        self.request = MindlessRequest(protocol=None)
+        transport = ConnectionLike()
+        protocol = RequestHandler()
+        protocol.makeConnection(transport)
+        self.request = MindlessRequest(protocol=protocol)
         self.error = None
-        self.request.error = lambda error: setattr(self, 'error', error)
 
     def tearDown(self):
         """Clean up."""
         self.error = None
         self.request = None
 
+    @inlineCallbacks
+    def test_disconnect_aborts_requests(self):
+        """Test that disconnection aborts outstanding requests."""
+
+        class OurException(RuntimeError):
+            """An exception class to look for."""
+
+        self.request.start()
+        self.request.protocol.connectionLost(Failure(OurException()))
+        try:
+            yield self.request.deferred
+        except OurException:
+            pass # passed
+        else:
+            self.fail("Expected to fail with the correct reason.")
+
     def test_default_process_message_basic(self):
         """_default_process_message maps errors to exceptions."""
+        self.patch(self.request, 'error',
+                   lambda error: setattr(self, 'error', error))
+
         message = protocol_pb2.Message()
         self.request._default_process_message(message)
 
@@ -106,6 +116,9 @@ class TestRequest(unittest.TestCase):
 
     def test_default_process_message_no_error_message(self):
         """_default_process_message maps errors to exceptions."""
+        self.patch(self.request, 'error',
+                   lambda error: setattr(self, 'error', error))
+
         message = protocol_pb2.Message()
         self.request._default_process_message(message)
 
@@ -115,6 +128,9 @@ class TestRequest(unittest.TestCase):
 
     def test_default_process_message(self):
         """_default_process_message maps errors to exceptions."""
+        self.patch(self.request, 'error',
+                   lambda error: setattr(self, 'error', error))
+
         for code_error, proto_error in errors._error_mapping.iteritems():
             message = protocol_pb2.Message()
             message.type = protocol_pb2.Message.ERROR
@@ -127,3 +143,46 @@ class TestRequest(unittest.TestCase):
             self.assertEqual(message, self.error.error_message)
 
             self.error = None
+
+    def test_start_returns_result(self):
+        """Test start method returns result."""
+        expected = object()
+        self.request.start_result = expected
+        actual = self.request.start()
+        self.assertEqual(expected, actual, "start must return _start's result")
+
+    def test_sendError(self):
+        """Test for sendError message."""
+        error_type = protocol_pb2.Error.QUOTA_EXCEEDED
+        comment = 'No more bytes for you.'
+        free_space_info = {'share_id':str(uuid.uuid4()), 'free_bytes':0}
+        self.request.id = 1
+        def check(message):
+            """Check the message"""
+            self.assertEqual(message.id, self.request.id)
+            self.assertEqual(message.type, protocol_pb2.Message.ERROR)
+            self.assertEqual(message.error.type,
+                             protocol_pb2.Error.QUOTA_EXCEEDED)
+            self.assertEqual(message.error.comment, comment)
+            self.assertEqual(message.free_space_info.free_bytes,
+                             free_space_info['free_bytes'])
+            self.assertEqual(message.free_space_info.share_id,
+                             free_space_info['share_id'])
+        self.patch(self.request, 'sendMessage', check)
+        self.request.sendError(error_type, comment=comment,
+                               free_space_info=free_space_info)
+
+
+class TestRequestResponse(TestRequest):
+    """Tests for RequestResponse."""
+
+    def setUp(self):
+        """Init."""
+        message = protocol_pb2.Message()
+        message.id = 1
+        transport = ConnectionLike()
+        protocol = RequestHandler()
+        protocol.makeConnection(transport)
+        self.request = MindlessRequestResponse(
+            protocol=protocol, message=message)
+        self.request.protocol.requests[message.id] = self.request
