@@ -30,7 +30,7 @@ from twisted.internet import reactor, defer
 from twisted.python import log
 
 from ubuntuone.storageprotocol import (
-    protocol_pb2, request, sharersp, volumes
+    protocol_pb2, request, sharersp, volumes, delta
 )
 
 log_debug = partial(log.msg, loglevel=logging.DEBUG)
@@ -55,6 +55,7 @@ class StorageClient(request.RequestHandler):
         self._account_info_callback = None
         self._volume_created_callback = None
         self._volume_deleted_callback = None
+        self._volume_new_generation_callback = None
 
         self.line_mode = True
 
@@ -178,6 +179,16 @@ class StorageClient(request.RequestHandler):
             vol_id = uuid.UUID(message.volume_deleted.volume)
             self._volume_deleted_callback(vol_id)
 
+    def handle_VOLUME_NEW_GENERATION(self, message):
+        """Handle incoming VOLUME_NEW_GENERATION."""
+        assert message.type == protocol_pb2.Message.VOLUME_NEW_GENERATION
+        if self._volume_new_generation_callback is not None:
+            volume = message.volume_new_generation.volume
+            if volume != request.ROOT:
+                volume = uuid.UUID(volume)
+            generation = message.volume_new_generation.generation
+            self._volume_new_generation_callback(volume, generation)
+
     def handle_BEGIN_CONTENT(self, message):
         """Accept and discard a misplaced BEGIN_CONTENT.
 
@@ -287,6 +298,21 @@ class StorageClient(request.RequestHandler):
         r.start()
         return r.deferred
 
+    def get_delta(self, share_id, from_generation=None, callback=None,
+                  from_scratch=False):
+        """Get a delta for a share_id
+
+        'share_id' is the share_id which we want to query.
+        'from_generation' is the generation which we are at.
+        'callback' can be specified to get deltas as they get instead of
+            getting them all at once at the end.
+        'from_scratch' at True means list all live nodes.
+
+        """
+        r = GetDelta(self, share_id, from_generation, callback, from_scratch)
+        r.start()
+        return r.deferred
+
     def get_free_space(self, share_id):
         """Get quota info for the given share (or the user's own space)."""
         r = FreeSpaceInquiry(self, share_id)
@@ -422,6 +448,13 @@ class StorageClient(request.RequestHandler):
         else:
             raise TypeError('callback for volume_deleted must be callable')
 
+    def set_volume_new_generation_callback(self, callback):
+        """Set the callback for volume new generation notification."""
+        if callable(callback):
+            self._volume_new_generation_callback = callback
+        else:
+            raise TypeError('callback for volume_new_gen must be callable')
+
     def create_share(self, node, share_to, name, access_level):
         """Create a share to other user.
 
@@ -864,7 +897,10 @@ class DeleteVolume(request.Request):
 
 
 class Unlink(request.Request):
-    """Unlink a node."""
+    """Unlink a node.
+
+    @ivar new_generation: the generation that the volume is at now
+    """
     # pylint: disable-msg=C0111
 
     def __init__(self, protocol, share, node_id):
@@ -878,6 +914,7 @@ class Unlink(request.Request):
         request.Request.__init__(self, protocol)
         self.share = share
         self.node = node_id
+        self.new_generation = None
 
     def _start(self):
         message = protocol_pb2.Message()
@@ -889,13 +926,17 @@ class Unlink(request.Request):
 
     def processMessage(self, message):
         if message.type == protocol_pb2.Message.OK:
+            self.new_generation = message.new_generation
             self.done()
         else:
             self._default_process_message(message)
 
 
 class Move(request.Request):
-    """Move a node."""
+    """Move a node.
+
+    @ivar new_generation: the generation that the volume is at now
+    """
 
     def __init__(self, protocol, share, node_id, new_parent_id, new_name):
         """Create the move request
@@ -913,6 +954,7 @@ class Move(request.Request):
         self.node_id = node_id
         self.new_parent_id = new_parent_id
         self.new_name = new_name
+        self.new_generation = None
 
     def _start(self):
         """Send MOVE."""
@@ -928,6 +970,7 @@ class Move(request.Request):
     def processMessage(self, message):
         """Process messages."""
         if message.type == protocol_pb2.Message.OK:
+            self.new_generation = message.new_generation
             self.done()
         else:
             self._default_process_message(message)
@@ -1083,7 +1126,10 @@ class BytesMessageProducer(object):
 
 
 class PutContent(request.Request):
-    """Put content request."""
+    """Put content request.
+
+    @ivar new_generation: the generation that the volume is at now
+    """
 
     def __init__(self, protocol, share, node_id,
                  previous_hash, new_hash, crc32, size,
@@ -1109,6 +1155,7 @@ class PutContent(request.Request):
         self.size = size
         self.deflated_size = deflated_size
         self.fd = fd
+        self.new_generation = None
 
     def _start(self):
         """Send PUT_CONTENT."""
@@ -1130,6 +1177,7 @@ class PutContent(request.Request):
                 self, self.fd, message.begin_content.offset)
             self.registerProducer(message_producer, streaming=True)
         elif message.type == protocol_pb2.Message.OK:
+            self.new_generation = message.new_generation
             self.done()
         elif message.type == protocol_pb2.Message.CANCELLED:
             self.error(request.RequestCancelledError("CANCELLED"))
@@ -1159,6 +1207,7 @@ class MakeObject(request.Request):
     @ivar new_id: the id of the node that was created (available upon success)
     @ivar new_parent_id: the parent id the node now exists under
     @ivar new_name: the name the node now exists under
+    @ivar new_generation: the generation that the volume is at now
 
     """
 
@@ -1193,6 +1242,7 @@ class MakeObject(request.Request):
             self.new_id = message.new.node
             self.new_parent_id = message.new.parent_node
             self.new_name = message.new.name
+            self.new_generation = message.new_generation
             self.done()
         else:
             self._default_process_message(message)
@@ -1241,7 +1291,10 @@ class ProtocolVersion(request.Request):
 
 
 class Authenticate(request.Request):
-    """Request to authenticate the user."""
+    """Request to authenticate the user.
+
+    @ivar session_id: the session id with the server.
+    """
 
     def __init__(self, protocol, auth_parameters):
         """Create an authentication request.
@@ -1252,6 +1305,7 @@ class Authenticate(request.Request):
         """
         request.Request.__init__(self, protocol)
         self.auth_parameters = auth_parameters
+        self.session_id = None
 
     def _start(self):
         """Send AUTH_REQUEST."""
@@ -1266,6 +1320,7 @@ class Authenticate(request.Request):
     def processMessage(self, message):
         """Handle messages."""
         if message.type == protocol_pb2.Message.AUTH_AUTHENTICATED:
+            self.session_id = message.session_id
             self.done()
         else:
             self._default_process_message(message)
@@ -1362,6 +1417,68 @@ class AccountInquiry(request.Request):
         else:
             self._default_process_message(message)
 
+class GetDelta(request.Request):
+    """Get a delta on a volume
+
+    @ivar end_generation: The generation the volume will be after aplying this
+                delta
+    @ivar full: True if all changes were included. False is there is some
+                later generations not included.
+    @ivar free_bytes: The free space of the volume.
+                Only a hint if full is False.
+    @ivar response: the list of deltanodes received or empty if called with
+                callback
+    """
+
+    def __init__(self, protocol, share_id, from_generation=None,
+                 callback=None, from_scratch=False):
+        """Generates a GET_DELTA message to the server.
+
+        @param protocol: the request handler
+        @param volume_id: the volume id
+        @param from_generation: the starting generation for the delta
+        @param from_scratch: request a delta with all live files
+
+        """
+        if from_generation is None and from_scratch is False:
+            raise TypeError("get_delta needs from_generation or from_scratch.")
+
+        request.Request.__init__(self, protocol)
+        self.share_id = str(share_id)
+        self.from_generation = from_generation
+        self.callback = callback
+        self.delta_message = dm = protocol_pb2.Message()
+        dm.type = protocol_pb2.Message.GET_DELTA
+        dm.get_delta.share = self.share_id
+        if not from_scratch:
+            dm.get_delta.from_generation = from_generation
+        dm.get_delta.from_scratch = from_scratch
+
+        self.response = []
+        self.end_generation = None
+        self.full = None
+        self.free_bytes = None
+        self.generation = None
+
+    def _start(self):
+        """Send GET_DELTA."""
+        self.sendMessage(self.delta_message)
+
+    def processMessage(self, message):
+        """Handle messages."""
+        if message.type == protocol_pb2.Message.DELTA_INFO:
+            info = delta.from_message(message)
+            if self.callback:
+                self.callback(info)
+            else:
+                self.response.append(info)
+        elif message.type == protocol_pb2.Message.DELTA_END:
+            self.end_generation = message.delta_end.generation
+            self.full = message.delta_end.full
+            self.free_bytes = message.delta_end.free_bytes
+            self.done()
+        else:
+            self._default_process_message(message)
 
 class ThrottlingStorageClient(StorageClient):
     """The throttling version of the StorageClient protocol."""
