@@ -373,7 +373,7 @@ class Request(object):
         message.id = self.id
         self.protocol.sendMessage(message)
 
-    def sendError(self, error_type, comment=None):
+    def sendError(self, error_type, comment=None, free_space_info=None):
         """create and send an error message of type error_type
 
         @param error_type: a value from protocol_pb2.Error
@@ -384,6 +384,9 @@ class Request(object):
         message.error.type = error_type
         if comment is not None:
             message.error.comment = comment
+        if free_space_info is not None:
+            message.free_space_info.share_id = free_space_info['share_id']
+            message.free_space_info.free_bytes = free_space_info['free_bytes']
         self.sendMessage(message)
 
     def _start(self):
@@ -442,13 +445,13 @@ class Request(object):
 
 
 class RequestResponse(Request):
-    """a request that is created in response to an incoming message.
+    """A request that is created in response to an incoming message.
 
     @ivar source_message: the message the generated this request.
     """
     # pylint: disable-msg=W0223
     def __init__(self, protocol, message):
-        """created a request response.
+        """Create a request response.
 
         @param protocol: the request handler.
         @param message: the source message.
@@ -457,12 +460,13 @@ class RequestResponse(Request):
         self.source_message = message
 
     def start(self):
-        """register this request and start response."""
-        Request.start(self, self.source_message.id)
+        """Register this request and start response."""
+        result = Request.start(self, self.source_message.id)
+        return result
 
 
 class Ping(Request):
-    """ Request to ping the other peer.
+    """Request to ping the other peer.
 
     @ivar rtt: will contain the round trip time when completed.
     """
