@@ -1,9 +1,10 @@
-# ubuntuone.storageprotocol.share.py -
+# ubuntuone.storageprotocol.sharersp.py -
 #           provides a handy class to use Shares
 #
 # Author: Facundo Batista <facundo@canonical.com>
+#         Guillermo Gonzalez <guillermo.gonzalez@canonical.com>
 #
-# Copyright 2009 Canonical Ltd.
+# Copyright 2009-2010 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License version 3,
@@ -28,7 +29,8 @@ class ShareResponse(object):
 
     @classmethod
     def from_params(cls, id, direction, subtree, share_name, other_username,
-                                   other_visible_name, accepted, access_level):
+                    other_visible_name, accepted, access_level,
+                    subtree_volume_id=None):
         """Creates the object from given parameters."""
         # pylint: disable-msg=W0201
         o = cls()
@@ -40,6 +42,11 @@ class ShareResponse(object):
         o.other_visible_name = other_visible_name
         o.accepted = accepted
         o.access_level = access_level
+        if direction == "to_me" and subtree_volume_id is not None:
+            raise ValueError("Shares with direction 'to_me' must not have"
+                             " a volume_id")
+        elif direction == "from_me":
+            o.subtree_volume_id = subtree_volume_id
         return o
 
     @classmethod
@@ -49,12 +56,17 @@ class ShareResponse(object):
         o = cls()
         o.id = uuid.UUID(msg.share_id)
         o.direction = volumes._direction_prot2nice[msg.direction]
-        o.subtree = msg.subtree
+        o.subtree = uuid.UUID(msg.subtree)
         o.name = msg.share_name
         o.other_username = msg.other_username
         o.other_visible_name = msg.other_visible_name
         o.accepted = msg.accepted
         o.access_level = volumes._access_prot2nice[msg.access_level]
+        if o.direction == "from_me":
+            if msg.subtree_volume_id:
+                o.subtree_volume_id = uuid.UUID(msg.subtree_volume_id)
+            else:
+                o.subtree_volume_id = None
         return o
 
     def dump_to_msg(self, msg):
@@ -67,6 +79,9 @@ class ShareResponse(object):
         msg.other_visible_name = self.other_visible_name
         msg.accepted = self.accepted
         msg.access_level = volumes._access_nice2prot[self.access_level]
+        if self.direction == "from_me":
+            if self.subtree_volume_id:
+                msg.subtree_volume_id = str(self.subtree_volume_id)
 
     def __str__(self):
         t = "Share %r [%s] (other: %s, access: %s, accepted: %s, id: %s)" % (
@@ -74,6 +89,7 @@ class ShareResponse(object):
                     self.access_level, self.accepted, self.id)
         return t
 
+
 class NotifyShareHolder(object):
     """This is a handy object to support all the fields of a share notify."""
 
